#include "xev.h"

#define USE_JOYSTICK

#ifdef USE_JOYSTICK
#include <fcntl.h>
#include <linux/joystick.h>

char joydev[256] = "/dev/js0";
int  joyinit = 0;
int  joyop   = 0;	/* 1:in joy operation */
int  joy_x_max = 0;
int  joy_y_max = 0;

#define JOY_NOP		0x00
#define JOY_UP		0x01
#define JOY_DOWN	0x02
#define JOY_LEFT	0x04
#define JOY_RIGHT	0x08
#define JOY_BT1		0x10
#define JOY_BT2		0x20
#endif /* USE_JOYSTICK */

PRIVATE int inkey = 0;

/*
 *   GetJoyStatus() : Get Joystick Status
 *
 *   IN  : Ȃ
 *   IN  : global : joydev : joystick device name (eg. /dev/js0)
 *   OUT : ret  (bit value)
 *           JOY_UP,_DOWN,_LEFT,_RIGHT,_BT1,_BT2
 *
 *   Copyright(C)1999, Moritaka Ogasawara.
 */
int GetJoyStatus()
{
    int status = 0;	/* no operation */
    int st;
#ifdef USE_JOYSTICK
    struct JS_DATA_TYPE js;
    int fd;

    /* open device file */
    if ((fd = open(joydev, O_RDONLY)) < 0) {
	printf("%s : open error(%d)\n", joydev, errno);
	return JOY_NOP;
    }

    st = read(fd, &js, JS_RETURN);
    if (st != JS_RETURN) {
	printf("%s : open error(%d)\n", joydev, errno);
	return JOY_NOP;
    }

    if (joyinit == 0) {
        joy_x_max = js.x*2;
	joy_y_max = js.y*2;
	joyinit = 1;
    }

    if (joy_x_max < js.x) joy_x_max = js.x;
    if (joy_y_max < js.y) joy_y_max = js.y;

    /* get paddle */
    if (js.x < joy_x_max/3)   status |= JOY_LEFT;	/* move left    */
    if (js.x > joy_x_max*2/3) status |= JOY_RIGHT;	/* move right   */
    if (js.y < joy_y_max/3)   status |= JOY_UP;		/* move up      */
    if (js.y > joy_y_max*2/3) status |= JOY_DOWN;	/* move down    */

    /* get button */
    if (js.buttons & 1) status |= JOY_BT1;		/* left button  */
    if (js.buttons & 2) status |= JOY_BT2;		/* right button */

    close(fd);
#endif /* USE_JOYSTICK */

    return status;
}

void KEClear()
{
      XEvent ev;

      while(XPending(d) > 0)
	XNextEvent(d,&ev);

      inkey = 0;
}

int CheckEvent()
{
      int i, k;
      KeySym ks;
      int oc_exp = 0,push_c = 0,push_s = 0 , push_q = 0, mask;

#ifdef USE_JOYSTICK
      int joyst = 0;
      if (joyop) {
          inkey = 0;
      }
      joyst = GetJoyStatus();
      if (joyst) {
          joyop = 1;
      }
#endif
      
      while (XPending(d) > 0){
	    mask = 0;
	    XNextEvent(d, &e);
	    switch (e.type){
		case Expose:
		  oc_exp = (1<<Exp);
		  break;
		  
		case KeyRelease:
		  switch (XKeycodeToKeysym(d, e.xkey.keycode, 0)){
		      case XK_Up: 
		      case XK_KP_8:
		      case XK_i:
			mask = (1<<Up); break;
		      case XK_Down: 
		      case XK_KP_2:
		      case XK_KP_5:
		      case XK_k:
		      case XK_m:
			mask = (1<<Down); break;
		      case XK_Left: 
		      case XK_KP_4:
		      case XK_j:
			mask = (1<<Left); break;
		      case XK_Right: 
		      case XK_KP_6:
		      case XK_l:
			mask = (1<<Right); break;
		      case XK_x: mask = (rev_key)?(1<<Kx):(1<<Kz); break;
		      case XK_z: mask = (rev_key)?(1<<Kz):(1<<Kx); break;
		      case XK_s: push_s = (1<<Ks); break;
		    case XK_q: push_q = (1<<Kq); break;
		      case XK_c: push_c = (1<<Kc); break;
		  }
		  inkey &= ~mask;
		  break;
		case KeyPress:
		  joyop = 0;	/* inkey clear flag by oga*/
		  switch (XKeycodeToKeysym(d, e.xkey.keycode, 0)){
		      case XK_Up: 
		      case XK_KP_8:
		      case XK_i:
			mask = (1<<Up); break;
		      case XK_Down: 
		      case XK_KP_2:
		      case XK_KP_5:
		      case XK_k:
		      case XK_m:
			mask = (1<<Down); break;
		      case XK_Left: 
		      case XK_KP_4:
		      case XK_j:
			mask = (1<<Left); break;
		      case XK_Right: 
		      case XK_KP_6:
		      case XK_l:
			mask = (1<<Right); break;
		      case XK_x: mask = (rev_key)?(1<<Kx):(1<<Kz); break;
		      case XK_z: mask = (rev_key)?(1<<Kz):(1<<Kx); break;
		  }
		  inkey |= mask;
		  break;
	    }
      }
#ifdef USE_JOYSTICK
      if (joyst) {
          inkey = 0;
          if (joyst & JOY_UP) {
              inkey |= (1<<Up);
          }
          if (joyst & JOY_DOWN) {
              inkey |= (1<<Down);
          }
          if (joyst & JOY_LEFT) {
              inkey |= (1<<Left);
          }
          if (joyst & JOY_RIGHT) {
              inkey |= (1<<Right);
          }
          if (joyst & JOY_BT1) {
              inkey |= ((rev_key)?(1<<Kx):(1<<Kz));
          }
          if (joyst & JOY_BT2) {
              inkey |= ((rev_key)?(1<<Kz):(1<<Kx));
          }
      }
#endif

      return inkey | push_s | push_q | push_c | oc_exp;
}



            


